defint a-z	' Die Variablen a-z werden als Integer definiert.

' Deklaration der vorkommenden Proceduren
declare sub Malen()
'declare sub Laden()
declare sub Scannen(y%,Tiefe() as integer)
declare sub Ausgabe(y%,Stereo() as integer)
declare sub MacheKorrel(Tiefe() as integer, Korrel() as integer, AugeAbst as single, Aufloesung as integer)
declare sub MacheSirds(Tiefe() as integer, Korrel() as integer)
declare function DotFarbe(byval x, byval y)

const Cnt=640               ' Pixel Anzahl in x-Richtung.
const maxy=480              ' Pixel Anzahl in y-Richtung.

const colnum=16 	    ' Anzahl der verfgbaren Farben.
const zSkal!=1.0/colnum     ' Normierungsfaktor auf Farbwerte zwischen 0 und 1.
const p!=1.0/3.0	    ' Tiefenskalierungsfaktor, der Wert ist empirisch ermittelt und gut.

dim Tiefe(Cnt) as integer
dim Korrel(Cnt) as integer
dim Stereo(Cnt) as integer


' Diese hier nicht explizit aufgefhrte Procedur ld das zu codierende Bild
' Wie wre es mit Ihrem eigenen "Punkt Punkt Komma Strich, fertig ist das Mondgesicht" ?
' sub Laden   ( diese Procedur ist optional)

' Hier wird das Mondgesicht gemalt
sub Malen

' Malen der Mundes.
  circle(320,240),100,5
  paint (320,240),5,5
  circle(320,220),100,0
  paint (320,215),0,0

' Malen der Augen.
  circle(250,100),40,2,,,1/3
  paint (250,100),2,2
  circle(390,100),40,2,,,1/3
  paint (390,100),2,2

' Malen der Nase.
  circle(320,200),60,7,,,2
  paint (320,200),7,7
' ... fertig ist das Mondgesicht.

' Malen des Steingrber Fachverlag Logos.
  line  (200,155)-(235,215),6,BF
  line  (205,175)-(225,195),0,BF
  line  (175,175)-(195,195),6,BF
end sub

' Diese Procedur ld die Farbinformation einer Bildzeile in das Feld Tiefe ein
sub Scannen(y%,Tiefe() as integer) static
for x=0 to Cnt-1
  Tiefe(x)=DotFarbe(x,y)
next x
end sub

' Die Ausgabe einer Zeile des SIRDS erfolgt hier
sub Ausgabe(y%,Stereo() as integer) static
for x=0 to Cnt-1
  pset(x,y),Stereo(x)	     ' Gibt den endgltigen SIRDS-Pixel aus.
next x
end sub

' Der Algorithmus fr die Korrelationsberechnung erfolgt mit MacheKorrel(). Dabei wird das von Scannen
' erzeugte Feld Tiefe() umgerechnet
sub MacheKorrel(Tiefe() as integer, Korrel() as integer, AugeAbst as single, Aufloesung as integer) static

' Das Feld Korrel() wird auf "nicht korreliert" gesetzt.
for x=0 to Cnt-1
  Korrel(x)=x
next x

E=AugeAbst!*Aufloesung        ' Der Augenabstand wird in Anzahl der Pixel umgerechnet.
konst!=2/(zSkal!*p!*E)

' Hier beginnt der Algorithmus.
for x=0 to Cnt-1

' Kernberechnung des Algorithmus.
  Vor=Tiefe(x)
  z!=Vor*zSkal!	     ' Normierung der Farbwerte zwischen 0 und 1.
  s=(1.0-p!*z!)*E/(2.0-p!*z!) ' Die Berechnung von S.
  Sl=x-S/2                    ' Berechnung des linken Stereopunktes.
  Sr=Sl+S  '                  ' Berechnung des rechten Stereopunktes.

' Weiter, wenn Stereopunkte beide im Abbildungsbereich (Bildschirm) liegen.
  if 0<=Sl AND Sr<Cnt then

' Teil des Algorithmus, der berprft, ob der betrachtete Punkt berhaupt frs Auge sichtbar ist.
' Hidden-Pixel Algorithmus. Siehe dazu Ausfhrung im Text.
    dx=1
    do
      zt=Vor+(2-p!*z!)*dx*konst!
      sichtbar=Tiefe(x-dx)<zt
      if sichtbar then
	sichtbar=Tiefe(x+dx)<zt
      end if
      dx=dx+1
    loop until Not sichtbar OR zt>zSkal!

' Teil des Algorithmus, der die Randbedingungen bercksichtigt.
' Lesen Sie dazu die Ausfhrungen im Text.
    if sichtbar then
      l=Korrel(Sl)
      while l<>Sl AND l<>Sr
        if l<Sr then
          Sl=l
          l=Korrel(Sl)
        else
          Korrel(Sl)=Sr
          Sl=Sr
          Sr=l         
          l=Korrel(Sl)
        end if
      wend
      Korrel(Sl)=Sr           ' Hier wird die Kernzuweisung gemacht,
                              ' wobei die Korrelierung festgelegt wird.
    end if
  end if
next x
end sub

' Diese Procedur setzt die im Feld Korrel() enthaltenen Korrelationsinformationen um und 
' setzt die Farbwerte der auszugebenen SIRDS-Zeile in das Feld Stereo() ein.
sub MacheSirds(Stereo() as integer, Korrel() as integer) static
for x=Cnt-1 to 0 step -1
  farbe=int((colnum-0)*rnd)    ' Die Zufallsfarbe wird generiert.

' Wenn keine Korrelierung vorliegt, dann wird einfach die Zufallsfarbe gesetzt.
  if Korrel(x)=x then
    Stereo(x)=farbe
  else
' Liegt Korrelierung vor, so setzte die gleiche Farbe wie die des anderen Stereopunktes.
    Stereo(x)=Stereo(Korrel(x))
  end if
next x
end sub

' ******************** Hier beginnt das Hauptprogramm ********************

cls
print " Punkt Punkt Punkt Komma Strich, fertig ist das Stereomondgesicht."
print " Nach drcken einer beliebigen Taste wird das Mondgesicht gemalt."
print " Ein weiteres Drcken einer beliebigen Taste startet die SIRDS-Berechnung."
print " Viel Spa beim Betrachten !"
sleep

' Der VGA-Modus wird initialisiert
screen 12
' call Laden (optional)
call Malen		       ' Das Mondgesicht wird gemalt.
sleep

' Mit dieser Schleife werden die einzelnen Zeilen abgearbeitet.
for y=0 to maxy-1
  call Scannen(y,Tiefe())		      ' Das Bild wird zeilenweise eingelesen.
  call MacheKorrel(Tiefe(),Korrel(),3,72)     ' Die Korrelierung der Zeile wird festgelegt.
  call MacheSirds(Stereo(),Korrel())          ' Die SIRDS-Farbwerte der Zeile werden gesetzt.
  call Ausgabe(y,Stereo())                    ' Die SIRDS-Zeile wird ausgegeben.
next y
sleep
