'============================================================
' Projekt        : Setup-Steuerung mit Hilfe von Makros
' Modulname      : MSGBOX.BAS
' Aufgabe        : Message-Boxen
' Copyright      : Arthur Burda
' Compiler       : Visual Basic 3.0 fr Windows
'============================================================
' l. nderung am : 26.07.1994
' Version        : 1.00
' Bemerkungen    : Erste Implementierung
'------------------------------------------------------------

Option Explicit

'------------------------------------------------------------
' Schalter
'------------------------------------------------------------

Global Const MB_OK = 0                 ' Nur Ok-Schalter
Global Const MB_OKCANCEL = 1           ' Ok- und Abbrechen-Schalter
Global Const MB_ABORTRETRYIGNORE = 2   ' Abbruch, Wiederholen, Ignorieren
Global Const MB_YESNOCANCEL = 3        ' Ja, Nein und Abbrechen
Global Const MB_YESNO = 4              ' Ja- und Nein-Schalter
Global Const MB_RETRYCANCEL = 5        ' Wiederholen- und Abbrechen-Schalter

'------------------------------------------------------------
' Icons
'------------------------------------------------------------

Global Const MB_ICONSTOP = 16          ' Stop-Icon (z.B. bei kritischem Fehler)
Global Const MB_ICONQUESTION = 32      ' Frage-Icon
Global Const MB_ICONEXCLAMATION = 48   ' Warnung-Icon
Global Const MB_ICONINFORMATION = 64   ' Information-Icon

'------------------------------------------------------------
' Sonstiges
'------------------------------------------------------------

Global Const MB_APPLMODAL = 0          ' Information, da die Applikation modal ist
Global Const MB_DEFBUTTON1 = 0         ' Erster Schalter ist Voreinstellung
Global Const MB_DEFBUTTON2 = 256       ' Zweiter Schalter ist Voreinstellung
Global Const MB_DEFBUTTON3 = 512       ' Dritter Schalter ist Voreinstellung
Global Const MB_SYSTEMMODAL = 4096     ' System modal

'------------------------------------------------------------
' MsgBox-Rckgabewerte
'------------------------------------------------------------

Global Const IDOK = 1                  ' Ok-Schalter gedrckt
Global Const IDCANCEL = 2              ' Abbrechen-Schalter gedrckt
Global Const IDABORT = 3               ' Abbruch-Schalter gedrckt
Global Const IDRETRY = 4               ' Wiederholen-Schalter gedrckt
Global Const IDIGNORE = 5              ' Ignorieren-Schalter gedrckt
Global Const IDYES = 6                 ' Ja-Schalter gedrckt
Global Const IDNO = 7                  ' Nein-Schalter gedrckt

'============================================================
' Routine : ShowCrErrorBox
'============================================================
' Aufgabe : Zeigt eine Fehler-Box an (kritischer Fehler), die
'           den Schalter "Ok" enthlt.
' Eingabe : Message = Meldung
' Ausgabe : keine
'------------------------------------------------------------
'
Sub ShowCrErrorBox (Message$)

    Dim BtnID%

    BtnID% = ShowMsgBox(CrErrorBoxCaption, Message$, MB_OK + MB_ICONSTOP)

End Sub

'============================================================
' Routine : ShowErrorBox
'============================================================
' Aufgabe : Zeigt eine Fehler-Box an, die die Schalter "Abbruch",
'           "Wiederholen", "Ignorieren" enthlt und liefert den
'           ID-Code des gedrckten Schalters zurck.
' Eingabe : Message    = Meldung
'           DefaultBtn = Nummer des voreingestellten Schalters
' Ausgabe : keine
' Return  : ID-Code des gedrckten Schalters
'------------------------------------------------------------
'
Function ShowErrorBox (Message$, DefaultBtn%) As Integer

    Select Case DefaultBtn%
        Case 1
            ShowErrorBox = ShowMsgBox(ErrorBoxCaption, Message$, MB_ABORTRETRYIGNORE + MB_ICONSTOP + MB_DEFBUTTON1)
        Case 2
            ShowErrorBox = ShowMsgBox(ErrorBoxCaption, Message$, MB_ABORTRETRYIGNORE + MB_ICONSTOP + MB_DEFBUTTON2)
        Case Else
            ShowErrorBox = ShowMsgBox(ErrorBoxCaption, Message$, MB_ABORTRETRYIGNORE + MB_ICONSTOP + MB_DEFBUTTON3)
    End Select

End Function

'============================================================
' Routine : ShowErrorBox2
'============================================================
' Aufgabe : Zeigt eine Fehletr-Box an, die nur den Schalter "Ok"
'           enthlt.
' Eingabe : Message = Meldung
' Ausgabe : keine
'------------------------------------------------------------
'
Sub ShowErrorBox2 (Message$)

    Dim BtnID%

    BtnID% = ShowMsgBox(ErrorBoxCaption, Message$, MB_OK + MB_ICONSTOP)

End Sub

'============================================================
' Routine : ShowInfoBox
'============================================================
' Aufgabe : Zeigt eine Info-Box, die den Schlater "Ok" ent-
'           hlt.
' Eingabe : Message = Meldung
' Ausgabe : keine
'------------------------------------------------------------
'
Sub ShowInfoBox (Message$)

    Dim BtnID%

    BtnID% = ShowMsgBox(InfoBoxCaption, Message$, MB_OK + MB_ICONINFORMATION)

End Sub

'============================================================
' Routine : ShowMsgBox
'============================================================
' Aufgabe : Zeigt eine Message-Box auf dem Bildschirm mit den
'           angegebenen Optionen an und liefert den ID-Code
'           des gedrckten Schalters als Ergebnis zrck.
' Eingabe : Caption = Kopfzeile
'           Message = Meldung
'           Options = Optionen
' Ausgabe : keine
' Return  : ID-Code des gedrckten Schalters
'------------------------------------------------------------
'
Function ShowMsgBox (Caption$, Message$, Options%) As Integer

    ShowMsgBox = MsgBox(Message$, Options%, Caption$)

End Function

'============================================================
' Routine : ShowOkCancelBox
'============================================================
' Aufgabe : Zeigt eine Message-Box mit den Schaltern "Ok"
'           und "Abbrechen" an und liefert IDOK zurck, wenn
'           der Ok-Schalter gedrckt wurde und IDCANCEL, wenn
'           der Abbrechen-Schalter gedrckt wurde.
' Eingabe : Caption    = Kopfzeile
'           Message    = Meldung
'           DefaultBtn = Nummer des voreingestellten Schalters
' Ausgabe : keine
' Return  : ID-Code des gedrckten Schalters
'------------------------------------------------------------
'
Function ShowOkCancelBox (Caption$, Message$, DefaultBtn%) As Integer

    Select Case DefaultBtn%
        Case 1
            ShowOkCancelBox = ShowMsgBox(Caption$, Message$, MB_OKCANCEL + MB_ICONQUESTION + MB_DEFBUTTON1)
        Case Else
            ShowOkCancelBox = ShowMsgBox(Caption$, Message$, MB_OKCANCEL + MB_ICONQUESTION + MB_DEFBUTTON2)
    End Select

End Function

'============================================================
' Routine : ShowYesNoBox
'============================================================
' Aufgabe : Zeigt eine Message-Box mit den Schaltern "Ja"
'           und "Nein" an und liefert IDYES zurck, wenn der
'           Ja-Schalter gedrckt wurde und IDNO, wenn der
'           Nein-Schalter gedrckt wurde.
' Eingabe : Caption    = Kopfzeile
'           Message    = Meldung
'           DefaultBtn = Nummer des voreingestellten Schalters
' Ausgabe : keine
' Return  : ID-Code des gedrckten Schalters
'------------------------------------------------------------
'
Function ShowYesNoBox (Caption$, Message$, DefaultBtn%)

    Select Case DefaultBtn%
        Case 1
            ShowYesNoBox = ShowMsgBox(Caption$, Message$, MB_YESNO + MB_ICONQUESTION + MB_DEFBUTTON1)
        Case Else
            ShowYesNoBox = ShowMsgBox(Caption$, Message$, MB_YESNO + MB_ICONQUESTION + MB_DEFBUTTON2)
    End Select

End Function

'============================================================
' Routine : ShowYesNoCancelBox
'============================================================
' Aufgabe : Zeigt eine Message-Box mit den Schaltern "Ja",
'           "Nein" und "Abbrechen" an und liefert den ID-Code
'           des gedrckten Schalters zurck.
' Eingabe : Caption    = Kopfzeile
'           Message    = Meldung
'           DefaultBtn = Nummer des voreingestellten Schalters
' Ausgabe : keine
' Return  : ID-Code des gedrckten Schalters
'------------------------------------------------------------
'
Function ShowYesNoCancelBox (Caption$, Message$, DefaultBtn%) As Integer

    Select Case DefaultBtn%
        Case 1
            ShowYesNoCancelBox = ShowMsgBox(Caption$, Message$, MB_YESNOCANCEL + MB_ICONQUESTION + MB_DEFBUTTON1)
        Case 2
            ShowYesNoCancelBox = ShowMsgBox(Caption$, Message$, MB_YESNOCANCEL + MB_ICONQUESTION + MB_DEFBUTTON2)
        Case Else
            ShowYesNoCancelBox = ShowMsgBox(Caption$, Message$, MB_YESNOCANCEL + MB_ICONQUESTION + MB_DEFBUTTON3)
    End Select

End Function

